import { EditorState } from "@codemirror/state";
import { EditorView, keymap, lineNumbers, highlightActiveLine, highlightActiveLineGutter } from "@codemirror/view";
import { defaultKeymap, indentWithTab, history, historyKeymap } from "@codemirror/commands";
import { StreamLanguage, defaultHighlightStyle, syntaxHighlighting, bracketMatching, foldGutter } from "@codemirror/language";
import { highlightSelectionMatches, searchKeymap, search } from "@codemirror/search";
import {autocompletion, completionKeymap, closeBrackets, closeBracketsKeymap} from "@codemirror/autocomplete"

import { sparql } from "@codemirror/legacy-modes/mode/sparql";
import { linter, lintGutter, LintSource } from "@codemirror/lint";
import { sparqlLinter } from "./extentions/sparql-linter";

const defaultExtentions = [
  keymap.of([
    indentWithTab,
    ...searchKeymap,
    ...historyKeymap
  ]), 
  lineNumbers(),
  StreamLanguage.define(sparql),
  syntaxHighlighting(defaultHighlightStyle, {fallback: true}),
  closeBrackets(),
  linter(sparqlLinter<LintSource>),
  lintGutter(),
  highlightActiveLine(),
  highlightSelectionMatches(),
  bracketMatching(),
  foldGutter(),
  history(),
  search({ top: true })
]

const defaultDoc = `PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>

SELECT * WHERE {
  ?sub ?pred ?obj .
 } LIMIT 10`

export function createSparqlEditor({ parent, onChange, value }) {
  const extensions: [any] = [ defaultExtentions] 
  const doc = value || defaultDoc;

  if(typeof onChange === 'function') {
    const updateListener = EditorView.updateListener.of((viewUpdate) => {
      if (viewUpdate.docChanged && typeof onChange === 'function') {
        const doc = viewUpdate.state.doc;
        const value = doc.toString();
        onChange(value, viewUpdate);
      }
    });
  
    extensions.push(updateListener)
  }



  return new EditorView({ parent, doc, extensions })
}